import importlib
from types import ModuleType
from typing import Any

from pydantic import Field
from pydantic import field_validator
from pydantic import model_validator
from pydantic_settings import BaseSettings
from pydantic_settings import SettingsConfigDict

from amsdal_server.configs.constants import DEVELOPMENT_ENVIRONMENT
from amsdal_server.configs.constants import PRODUCTION_ENVIRONMENT
from amsdal_server.configs.constants import TESTING_ENVIRONMENT
from amsdal_server.configs.constants import check_force_test_environment


class Settings(BaseSettings):
    model_config = SettingsConfigDict(
        case_sensitive=True,
        env_prefix='AMSDAL_SERVER_',
        env_file='.env',
        env_file_encoding='utf-8',
    )

    APP_NAME: str = 'amsdal-server'
    DEBUG: bool = False
    IS_DOCS_ENABLED: bool = True
    ENVIRONMENT: str = ''

    HOST: str = '0.0.0.0'  # noqa: S104
    PORT: int = 80

    AUTHORIZATION_HEADER: str = 'Authorization'

    INSTALLED_APPS: list[str] = Field(
        default=[
            'amsdal_server.apps.classes',
            'amsdal_server.apps.objects',
            'amsdal_server.apps.transactions',
            'amsdal_server.apps.common',
            'amsdal_server.apps.healthcheck',
        ]
    )

    MIDDLEWARES: list[str | tuple[str, dict[str, Any]]] = Field(
        default=[
            'amsdal_server.apps.common.middlewares.response_event.ResponseEventMiddleware',
            (
                'starlette.middleware.cors.CORSMiddleware',
                {
                    'allow_origins': ['*'],
                    'allow_methods': ['*'],
                    'allow_headers': ['*'],
                    'allow_credentials': True,
                },
            ),
            (
                'starlette.middleware.authentication.AuthenticationMiddleware',
                {
                    'backend': 'amsdal_server.apps.common.authentication.AmsdalAuthenticationBackend',
                },
            ),
            'amsdal_server.apps.common.middlewares.logger.LoggerMiddleware',
        ]
    )

    LOGGING_FORMAT: str = (
        '%(levelname)-8s [%(asctime)s] %(message)s %(name)s.%(funcName)s:%(lineno)d [%(correlation_id)s]'
    )
    LOGGING_LOG_LEVEL: str = 'INFO'
    LOGGING_HANDLERS: list[str] = Field(default=['default'])
    LOGGING: dict[str, Any] = Field(default_factory=dict)

    @field_validator('ENVIRONMENT')
    @classmethod
    def validate_environment(cls, value: str | None) -> str:
        valid_environments = (
            TESTING_ENVIRONMENT,
            DEVELOPMENT_ENVIRONMENT,
            PRODUCTION_ENVIRONMENT,
        )

        if not value:
            value = DEVELOPMENT_ENVIRONMENT

        if value not in valid_environments:
            msg = f'Invalid environment. Valid environments are: {valid_environments}'
            raise ValueError(msg)

        return check_force_test_environment(default=value)

    @field_validator('INSTALLED_APPS')
    @classmethod
    def load_installed_apps(cls, values: list[str]) -> list[ModuleType]:
        return [importlib.import_module(module) for module in values]

    @model_validator(mode='after')
    def validate_logging(self) -> 'Settings':
        self.LOGGING = {
            'version': 1,
            'disable_existing_loggers': False,
            'formatters': {
                'default': {
                    'format': self.LOGGING_FORMAT,
                    'style': '%',
                    'use_colors': True,
                },
                'json': {
                    'format': '[%(correlation_id)s] %(message)s',
                    'class': 'pythonjsonlogger.jsonlogger.JsonFormatter',
                },
            },
            'filters': {},
            'handlers': {
                'default': {
                    'class': 'logging.StreamHandler',
                    'filters': [],
                    'level': self.LOGGING_LOG_LEVEL,
                    'formatter': 'default',
                },
                'json': {
                    'class': 'logging.StreamHandler',
                    'filters': [],
                    'level': self.LOGGING_LOG_LEVEL,
                    'formatter': 'json',
                },
                'null': {
                    'class': 'logging.NullHandler',
                },
            },
            'loggers': {
                # app logger
                'amsdal_server': {
                    'level': self.LOGGING_LOG_LEVEL,
                    'propagate': True,
                },
            },
        }

        for logger in self.LOGGING['loggers'].values():
            if 'handlers' not in logger:
                logger['handlers'] = self.LOGGING_HANDLERS

        return self


settings = Settings()
